
import java.awt.*;

import javax.swing.*;
import java.awt.event.*;
import java.util.Vector;
import gates.*;

class GraphicalInput {
	GraphicalGate gate;
	int pos;
	GraphicalInput(GraphicalGate gate,int pos) {
		this.gate = gate;
		this.pos = pos;
	}
}
class GateCanvas extends Canvas implements MouseListener, MouseMotionListener {
	private Vector buttons;

	private Vector circuit;

	private GraphicalGate selectedGate;
	private GraphicalGate selectedOutput;
	private GraphicalInput selectedInput;

	private Point cursor;

	GateCanvas() {
		buttons = new Vector();
		int x = 5 + GraphicalGate.getWidth() / 2;
		buttons.addElement(new GraphicalAnd(x, 35, new AndGate()));
		x += GraphicalGate.getWidth() + 5;
		buttons.addElement(new GraphicalOr(x, 35, new OrGate()));
		x += GraphicalGate.getWidth() + 5;
		buttons.addElement(new GraphicalNot(x, 35, new NotGate()));
		x += GraphicalGate.getWidth() + 5;
		buttons.addElement(new GraphicalValue(x,35,new ValueGate(true)));
		x += GraphicalGate.getWidth() + 5;
		buttons.addElement(new GraphicalValue(x,35,new ValueGate(false)));
		x += GraphicalGate.getWidth() + 5;
		buttons.addElement(new GraphicalPassthrough(x,35,new PassthroughGate()));
		circuit = new Vector();
		addMouseListener(this);
		addMouseMotionListener(this);
	}

	public void paint(Graphics g) {
		for (int i = 0; i < buttons.size(); ++i) {
			((GraphicalGate) buttons.elementAt(i)).draw(g);
		}
		for (int i = 0; i < circuit.size(); ++i) {
			((GraphicalGate) circuit.elementAt(i)).draw(g);
		}
		if (selectedInput != null) {
			Point a = selectedInput.gate.getInputPos(selectedInput.pos);
			g.drawLine((int) a.getX(),(int) a.getY(),(int) cursor.getX(),(int) cursor.getY());
		}
		if (selectedOutput != null) {
			Point a = selectedOutput.getOutputPos();
			g.drawLine((int) a.getX(),(int) a.getY(),(int) cursor.getX(),(int) cursor.getY());
		}
		// Highlights the input where the cursor is
		GraphicalInput in = getInputUnder(cursor);
		if (in != null) {
			in.gate.highlightInput(g,in.pos);
		}
		GraphicalGate out = getOutputUnder(cursor);
		if (out != null) {
			out.highlightOutput(g);
		}
	}
	public GraphicalInput getInputUnder(Point p) {
		for (int i = 0; i < circuit.size(); ++i) {
			GraphicalGate g = (GraphicalGate) circuit.elementAt(i);
			if (g.getGate().getNbInputs() > 0
					&& p.distance(g.getInputPos(0)) < 8) {
						return new GraphicalInput(g,0);
			}	
			if (g.getGate().getNbInputs() > 1 && p.distance(g.getInputPos(1)) < 3) return new GraphicalInput(g,1);
		}
		return null;
	}		
	public GraphicalGate getOutputUnder(Point p) {
		for (int i = 0; i < circuit.size(); ++i) {
			GraphicalGate g = (GraphicalGate) circuit.elementAt(i);
			if (p.distance(g.getOutputPos()) < 8) return g;
		}
		return null;
	}		
	
	public GraphicalGate getGateUnder(Point p) {
		for (int i = 0; i < circuit.size(); ++i) {
			GraphicalGate g = (GraphicalGate) circuit.elementAt(i);
			if (g.isBodyClicked((int) p.getX(),(int) p.getY())) return g;
		}
		return null;		
	}
	private Point pointOf(MouseEvent e) { return new Point((int) e.getX(),(int) e.getY()); }
	public void mousePressed(MouseEvent e) {
		cursor = new Point(e.getX(), e.getY());
		selectedInput = getInputUnder(pointOf(e));
		selectedOutput = getOutputUnder(pointOf(e));
		if (selectedInput == null && selectedOutput == null) selectedGate = getGateUnder(pointOf(e));
		if (selectedGate == null) {
			for (int i = 0; i < buttons.size(); ++i) {
				GraphicalGate g = (GraphicalGate) buttons.elementAt(i);
				if (g.isBodyClicked(e.getX(), e.getY())) {
					GraphicalGate h = g.getClone();
					h.setXY(e.getX(), e.getY());
					circuit.addElement(h);
					selectedGate = h;
					break;
				}
			}
		}
		repaint();
	}

	public void mouseReleased(MouseEvent e) {
		if (selectedGate != null) selectedGate.setXY(e.getX(), e.getY());
		if (selectedInput != null) selectedOutput = getOutputUnder(pointOf(e));
		if (selectedOutput != null && selectedInput == null) selectedInput = getInputUnder(pointOf(e));
		if (selectedInput != null && selectedOutput != null) {
			selectedInput.gate.gateOnInputs.setElementAt(selectedOutput,selectedInput.pos);
			Gate.connect(selectedOutput.getGate(),selectedInput.gate.getGate(),selectedInput.pos);
		}
		selectedGate = null;
		selectedInput = null;
		selectedOutput = null;
		repaint();
	}

	public void mouseClicked(MouseEvent e) {
		if (e.getClickCount() == 2) {
			GraphicalGate g = getGateUnder(pointOf(e));
			if (g != null) g.reactToDoubleClick();
		}
		repaint();
	}

	public void mouseEntered(MouseEvent e) {
	}

	public void mouseExited(MouseEvent e) {
	}

	public void mouseDragged(MouseEvent e) {
		if (selectedGate != null) selectedGate.setXY(e.getX(), e.getY());		
		cursor = new Point(e.getX(), e.getY());
		repaint();
	}
	public void mouseMoved(MouseEvent e) {
		cursor = new Point(e.getX(), e.getY());
		repaint();
	}
}
/**
 * @author Xavier Dcoret
 * 
 * TODO To change the template for this generated type comment go to Window -
 * Preferences - Java - Code Style - Code Templates
 */

public class Main {
	public static void main(String[] args) {
		// Create the graphical widgets
		JFrame fr = new JFrame("Graph Java");
		fr.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		fr.setSize(500, 500);

		fr.getContentPane().add(new GateCanvas());
		fr.setVisible(true);
	}
}